/*
---

name: Youjoomla Mouseover Effect

description: allows mouseover/out effects on image stories

author: Youjoomla

license: This file is NOT licensed under any BSD or MIT style license.
         Unauthorised use, editing, distribution is not allowed.

copyright: Youjoomla.com 2012

version: 1.0

requires:
  - Core/String
  - Core/Fx
  - Core/Event
  - Core/Element
  - Core/Array
  - Core/Class
  - ScaleCSS3

provides: MooverEffect

...
*/
(function() {

    // allow it to run on older mootools...
    // start compat layer - can remove if mootools 1.3+
    if (!Object.merge) {
        Object.merge = $merge;
    }

    if (!Object.clone) {
        Object.clone = function(obj) {
            // serialize a simple object
            return JSON.decode(JSON.encode(obj));
        };
    }
    // end compat layer

    var MooverEffect = this.MooverEffect = new Class({
        // main class definition

        Implements: [Options, Events],

        // default options
        options: {
            mask: "span.catItemImage img,a.item_image img,span.itemImage img,a.subCategoryImage img,a.moduleItemImage img",
            overlayZen: "div",
            overlayClass: "roe-overlay",
            toggleClass: "displayNone",
            title: "title",
            titleZen: "div",
            titleClass: "roe-title",
            fx: {
                genericsOverlay: {
                    duration: 300,
                    link: 'cancel'
                },
                genericsTitle: {
                    duration: 800,
                    link: 'cancel',
                    transition: Fx.Transitions.Sine.easeOut
                },
                onOverlay: {
                    opacity: .6
                },
                offOverlay: {
                    opacity: 0
                },
                onTitle: {
                    opacity: [0.1,0.9]
                },
                titleStartOffset: 0
            }
        },

        initialize: function(el, options) {
            // constructor
            this.setOptions(options);
            //this.detectTransforms();
            this.element = document.id(el) || document.id(document.body);

            this.elements = this.element.getElements(this.options.mask);
            this.attachEffect();
            this.fireEvent("ready");
        },

        attachEffect: function() {
            // generic method that preps the dom
            var self = this;

            var titleAnimObj = this.options.fx.onTitle;

            // store a reference to bound funcs
            this.boundEvents = {
                mouseenter: function(e) {
                    var shim = this.retrieve("shim"), title = shim.retrieve("title");
                    shim.removeClass(self.options.toggleClass);
                    titleAnimObj.marginTop = title.retrieve("offset");
                    title.removeClass(self.options.toggleClass).setStyle("opacity", 0).morph(titleAnimObj);
                    shim.morph(shim.retrieve("morphOn"));

                },
                mouseleave: function(e) {
                    var shim = this.retrieve("shim"), title = shim.retrieve("title");
                    title.addClass(self.options.toggleClass);
                    shim.morph(shim.retrieve("morphOff"));
                }
            };
            this.elements.each(this.attachEvent.bind(this));
        },

        attachEvent: function(el) {
            // create shim and attach mouseover events to a single element
            this.createShim(el);
            el.getParent().addEvents(this.boundEvents);
        },

        removeEffects: function() {
            // uninstall effect from all els.
            var self = this;
            this.elements.each(function(el) {
                el.retrieve("shim") && el.retrieve("shim").destroy();
                el.retrieve("title") && el.retrieve("title").destroy();
                el.getParent().removeEvents(self.boundEvents);
            });
        },

        createShim: function(el) {
            // prep dom
            var size = el.getSize(), half = {
                x: size.x / 2,
                y: size.y / 2
            };

            var shim = new Element(this.options.overlayZen, {
                "class": this.options.overlayClass,
                styles: {
                    width: size.x,
                    height: size.y,
                    marginLeft: half.x,
                    marginTop: half.y,
                    opacity: 0,
                    backgroundPosition: [size.y-18, "px ", size.x-18, "px"].join("")
                }
            }).inject(el, "before");

            var parent = el.getParent();

            parent.store("shim", shim);

            shim.set("morph", this.options.fx.genericsOverlay);

            var objOn = Object.merge(Object.clone(this.options.fx.onOverlay), {
                width: [0, size.x],
                height: [0, size.y],
                marginLeft: 0,
                marginTop: 0
            }), objOff = Object.merge(Object.clone(this.options.fx.offOverlay), {
                width: 0,
                height: 0,
                marginLeft: half.x,
                marginTop: half.y
            });

            shim.store("morphOn", objOn).store("morphOff", objOff);

 			var title = new Element(this.options.titleZen, {
                html: parent.get(this.options.title),
                "class": this.options.titleClass,
                styles: {
                    opacity: 0
                }
            }).inject(parent, "top");

            var titleSize = title.getSize().y;
            title.setStyles({
                opacity: 0,
                width: size.x - 10,
                height: titleSize
            }).set("morph", this.options.fx.genericsTitle).store("offset", [this.options.fx.titleStartOffset, half.y - titleSize / 2]);

            shim.addClass(this.options.toggleClass).setStyles({
                width: 0,
                height: 0
            });

            shim.store("title", title);
        }
    });


})(); // end closure.
window.addEvent("load", function() {
    new MooverEffect();
});